#pragma once

#define VULKAN_HPP_NO_CONSTRUCTORS
#define VULKAN_HPP_NO_EXCEPTIONS
#include <vulkan/vulkan.hpp>

// includes for child classes
#include "vulkan_util.hpp"

#define LOG_SUBLOGS
#include <gz-util/log.hpp>

#include "texture_manager.hpp"
#include "vertex.hpp"
#include "vulkan_allocator.hpp"



#include <vector>

namespace gz::vlk {
    /// Defined in texture_manager.hpp
    class TextureManager;
    /// Defined in vulkan_instance.hpp
    class VulkanInstance;

    class Renderer {
        public:
            Renderer(VulkanInstance& instance, TextureManager& textureManager) : vk(instance), textureManager(textureManager) {};
        protected:
            /**
             * @brief Cleanup vulkan resources held by this class
             * @details
             *  - destroy commandBuffers
             *  - destroy(free) indexBuffer(Memory)
             *  - destroy(free) vertexBuffer(Memory)
             */
            void cleanup_();
            VulkanInstance& vk;
            TextureManager& textureManager;
        /**
         * @name Image (render targets)
         * @details
         *  The images are used as render targets, `#ifdef GZ_RENDERER_OWN_RENDER_IMAGES`
         *  
         *  After rendering, the current image gets blitted onto the current swap chain image.
         *  These resources need to be created and cleaned up by the subclass
         *  @todo
         */
            /// @{
            std::vector<vk::Image> renderImages;
            std::vector<MemoryInfo> renderImageMemory;
            std::vector<vk::ImageView> renderImageViews;
            vk::Sampler imageSampler;
            /// @}

        /**
         * @name Descriptors
         * @details
         *  The descriptor pool has to be created by the subclass.
         */
            vk::DescriptorPool descriptorPool;
            void createDescriptorSetLayoutSampler(uint32_t binding);
	        void createDescriptorSetSampler();
            vk::DescriptorSetLayout sampleFromRenderImageDSL;
            std::vector<vk::DescriptorSet> sampleFromRenderImageDS;

            std::vector<vk::CommandBuffer> commandBuffers;
            /// On device local memory
            vk::Buffer vertexBuffer;
            MemoryInfo vertexBufferMemory;
            vk::DeviceSize vertexBufferSize;
            vk::Buffer indexBuffer;
            MemoryInfo indexBufferMemory;
            vk::DeviceSize indexBufferSize;
    };  // class RendererBase
}  // namespace gz::vlk
