#pragma once 

#include "texture_atlas.hpp"

#include "stb_image.h"

#include <gz-util/util/string.hpp>
#include <gz-util/log.hpp>
#include <glm/fwd.hpp>

#include <unordered_map>

namespace gz::vlk {
    struct TextureInfo {
        uint32_t atlas;
        glm::vec2 texCoordTopLeft;
        glm::vec2 texCoordBottomRight;
    };
    /// Defined in vulkan_instance.hpp
    class VulkanInstance;
    class TextureManager {
        public:
            /**
             * @brief Create a texture manager
             * @details 
             *  -# @ref VulkanInstance::registerCleanupCallback "register" @ref cleanup() "cleanup callback"
             *  -# @ref createDescriptor  "create descriptor set layout, pool and set
             */
            TextureManager(VulkanInstance& instance);
            void getTexCoords(const std::string& textureName, glm::vec2& texCoords);
			const vk::DescriptorSet& getDescriptorSet() const { return descriptorSet; }
			const vk::DescriptorSetLayout& getDescriptorSetLayout() const { return descriptorSetLayout; }
            // TODO
            /// @todo take texture as argument
            const TextureAtlas& getTextureAtlas() const { return atlases.at(0); }
            uint32_t getAtlasCount() const { return static_cast<uint32_t>(atlases.size()); }


        private:
            void cleanup();
            void loadTextureFromFile(const std::string& textureName);
            // TODO index currently has no meaning
            std::vector<TextureAtlas> atlases;
            /**
             * @brief Contains textureName : TextureInfo pairs
             */
            util::unordered_string_map<TextureInfo> textureInfos;
            VulkanInstance& vk;
            Log tLog;
            /**
             * @name Create desciptors
             * @details These functions create a desciptor with bindings for
             * -# Array of combined image samplers (DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER) at binding 0. Array length is atlas count
             *  @{
             */
            vk::DescriptorSetLayout descriptorSetLayout;
            vk::DescriptorPool descriptorPool;
            vk::DescriptorSet descriptorSet;
            /* std::vector<vk::DescriptorSet> descriptorSets; */
            /**
             * @brief Create a descriptor set, layout and pool
             * @details
             *  Bindings:
             *  -0. texture sampler array COMBINED_IMAGE_SAMPLER[atlasCount]
             */
            void createDescriptorResources();
            /* const uint32_t bindingCombinedImageSampler = 1; */

            /**
             * @}
             */


    };
}  // namespace gz::vlk
