#include "vertex.hpp"

#define VULKAN_HPP_NO_CONSTRUCTORS
#include <vulkan/vulkan.hpp>
#include <gz-util/util/string_conversion.hpp>

#include <glm/gtc/matrix_transform.hpp>

namespace gz::vlk {
const uint32_t BINDING = 0;

template<GLM_vec2_or_3 PosVec>
vk::Format getVkFormat() {
    if (std::same_as<PosVec, glm::vec3>) {
        return vk::Format::eR32G32B32Sfloat;
    }
    else if (std::same_as<PosVec, glm::vec2>) {
        return vk::Format::eR32G32Sfloat;
    }
}


//
// 3D VERTEX
//
    template<GLM_vec2_or_3 PosVec>
	std::string Vertex<PosVec>::toString() const {
        return "pos: <" + gz::toString(pos) + ", color: " + gz::toString(color) + ", texCoords: " + gz::toString(texCoord) + ">";
	};


    template<GLM_vec2_or_3 PosVec>
    vk::VertexInputBindingDescription Vertex<PosVec>::getBindingDescription() {
        vk::VertexInputBindingDescription bindingD {
            .binding = BINDING,
            .stride = sizeof(Vertex<PosVec>),
            .inputRate = vk::VertexInputRate::eVertex,
        };
        return bindingD;
    }

    template<GLM_vec2_or_3 PosVec>
    std::array<vk::VertexInputAttributeDescription, 3> Vertex<PosVec>::getAttributeDescriptions() {
        std::array<vk::VertexInputAttributeDescription, 3> inputAttributeD{};
        inputAttributeD[0].binding = BINDING;
        inputAttributeD[0].location = 0;
        inputAttributeD[0].format = getVkFormat<PosVec>();
        inputAttributeD[0].offset = offsetof(Vertex, pos);
        inputAttributeD[1].binding = BINDING;
        inputAttributeD[1].location = 1; 
        inputAttributeD[1].format = vk::Format::eR32G32B32Sfloat;
        inputAttributeD[1].offset = offsetof(Vertex, color);
        inputAttributeD[2].binding = BINDING;
        inputAttributeD[2].location = 2;
        inputAttributeD[2].format = vk::Format::eR32G32Sfloat;
        inputAttributeD[2].offset = offsetof(Vertex, texCoord);

        return inputAttributeD;
    }

    template<GLM_vec2_or_3 PosVec>
    bool Vertex<PosVec>::operator==(const Vertex<PosVec>& other) const {
        return pos == other.pos and color == other.color and texCoord == other.texCoord;
    }

	template std::string Vertex<glm::vec2>::toString() const;
	template std::string Vertex<glm::vec3>::toString() const;
    template vk::VertexInputBindingDescription Vertex<glm::vec2>::getBindingDescription();
    template vk::VertexInputBindingDescription Vertex<glm::vec3>::getBindingDescription();
    template std::array<vk::VertexInputAttributeDescription, 3> Vertex<glm::vec2>::getAttributeDescriptions();
    template std::array<vk::VertexInputAttributeDescription, 3> Vertex<glm::vec3>::getAttributeDescriptions();
    template bool Vertex<glm::vec2>::operator==(const Vertex<glm::vec2>& other) const;
    template bool Vertex<glm::vec3>::operator==(const Vertex<glm::vec3>& other) const;

} // namespace gz::vlk
